@@ -53,7 +53,9 @@ class Agent(BaseAction):
                 # - then wait for the JobRun to be in_progress from the API
 
                 active_reservation_id = None
+                hardware = None
                 job_run_data: dict = {}
+                active_reservation_data: dict = {}
 
                 if RUNNING_IN_CONTAINER and job_run_id:
                     job_run_result = self.primitive.jobs.get_job_run(id=job_run_id)
@@ -81,16 +83,51 @@ class Agent(BaseAction):
                     if RUNNING_IN_CONTAINER:
                         logger.info("Running in container, exiting due to no JobRun.")
                         break
-                    logger.debug("No pending Job Run found, sleeping...")
+                    logger.debug("No pending Job Run found. [sleeping 5 seconds]")
                     sleep(5)
                     continue
 
                 logger.debug("Found pending Job Run")
-                logger.debug(f"Job Run ID: {job_run_data.get('id')}")
-                logger.debug(f"Job Name: {job_run_data.get('job').get('name')}")
+                logger.debug(f"Job Run ID: {job_run_data.get('id', 'No Job ID Found')}")
+                logger.debug(
+                    f"Job Name: {job_run_data.get('job', {}).get('name', 'No Job Name Found')}"
+                )
 
                 job_run_status = job_run_data.get("status", None)
 
+                hardware_id = hardware.get("id", None) if hardware else None
+                execution_hardware_id = None
+                if job_run_data:
+                    execution_hardware = job_run_data.get("executionHardware", None)
+                    execution_hardware_id = (
+                        execution_hardware.get("id", None)
+                        if execution_hardware
+                        else None
+                    )
+                target_hardware_id = None
+
+                if (
+                    hardware_id is not None
+                    and execution_hardware_id is not None
+                    and (hardware_id != execution_hardware_id)
+                ):
+                    logger.info(
+                        f"Job Run {job_run_id} is being executed by the controller. Agent may stop. [sleeping 5 seconds]"
+                    )
+                    sleep(5)
+                    continue
+
+                # only set the target hardware if there are multiple hardware in the reservation. else its just running on itself
+                if (
+                    active_reservation_data
+                    and active_reservation_id
+                    and len(active_reservation_data.get("hardware", [])) > 1
+                ):
+                    for hardware in active_reservation_data.get("hardware", []):
+                        if hardware.get("id", None) != execution_hardware_id:
+                            target_hardware_id = hardware.get("id", None)
+                            break
+
                 while True:
                     if job_run_status == "pending":
                         # we are setting to request_in_progress here which puts a started_at time on the JobRun in the API's database
@@ -116,7 +153,7 @@ class Agent(BaseAction):
                         or job_run_status == "completed"
                     ):
                         logger.error(
-                            f"JobRun {job_run_data.get('name')} is already completed with conclusion {job_run_data.get('jobRun', {}).get('conclusion', 'unknown')}. Exiting."
+                            f"JobRun {job_run_data.get('jobRunNumber')} is already completed with status {job_run_data.get('status', 'unknown')} conclusion {job_run_data.get('conclusion', 'unknown')}. Exiting."
                         )
                         break
 
@@ -135,6 +172,7 @@ class Agent(BaseAction):
                         runner = Runner(
                             primitive=self.primitive,
                             job_run=job_run_data,
+                            target_hardware_id=target_hardware_id,
                         )
                         runner.setup()
                     except Exception as exception:
@@ -164,9 +202,9 @@ class Agent(BaseAction):
                         # This should probably eventually be another daemon?
                         uploader.scan()
 
-                    if RUNNING_IN_CONTAINER:
-                        logger.info("Running in container, exiting after job run")
-                        break
+                if RUNNING_IN_CONTAINER:
+                    logger.info("Running in container, exiting after job run")
+                    break
 
                 sleep(5)
         except KeyboardInterrupt:
