@@ -11,6 +11,7 @@ from loguru import logger
 
 from primitive.graphql.sdk import create_requests_session
 from primitive.utils.actions import BaseAction
+from ..graphql.utility_fragments import operation_info_fragment
 
 from ..utils.auth import create_new_session, guard
 from ..utils.chunk_size import calculate_optimal_chunk_size, get_upload_speed_mb
@@ -62,6 +63,8 @@ class Files(BaseAction):
         chunk_size: int,
         number_of_parts: int,
         is_public: bool = False,
+        organization_id: Optional[str] = None,
+        is_global: Optional[bool] = False,
     ):
         mutation = gql(pending_file_create_mutation)
         input = {
@@ -73,12 +76,21 @@ class Files(BaseAction):
             "isPublic": is_public,
             "chunkSize": chunk_size,
             "numberOfParts": number_of_parts,
+            "isGlobal": is_global,
         }
+        if organization_id:
+            input["organizationId"] = organization_id
         variables = {"input": input}
         result = self.primitive.session.execute(
             mutation, variable_values=variables, get_execution_result=True
         )
-        return result.data.get("pendingFileCreate")
+        data = result.data.get("pendingFileCreate")
+
+        messages = data.get("messages")
+        if messages:
+            raise Exception("\n".join([message.get("message") for message in messages]))
+
+        return data
 
     def _update_file_status(
         self,
@@ -124,15 +136,41 @@ class Files(BaseAction):
         return result
 
     @guard
-    def get_file(self, file_id: Optional[str] = None):
+    def files(
+        self,
+        file_id: Optional[str] = None,
+        file_name: Optional[str] = None,
+        organization_id: Optional[str] = None,
+        organization_slug: Optional[str] = None,
+        global_files: Optional[bool] = None,
+    ):
         query = gql(files_list)
 
         filters = {}
+        if not organization_id and not organization_slug and not global_files:
+            whoami_result = self.primitive.auth.whoami()
+            default_organization = whoami_result.data["whoami"]["defaultOrganization"]
+            organization_id = default_organization["id"]
+            logger.info(
+                f"Using default organization ID: {default_organization.get('slug')} ({organization_id})"
+            )
+        if organization_slug and not organization_id:
+            organization = self.primitive.organizations.get_organization(
+                slug=organization_slug
+            )
+            organization_id = organization.get("id")
+
+        if organization_id:
+            filters["organization"] = {"id": organization_id}
         if file_id:
-            filters["id"] = {"exact": file_id}
+            filters["id"] = file_id
+        if file_name:
+            filters["fileName"] = {"exact": file_name}
+        if global_files:
+            filters["isGlobal"] = {"exact": True}
 
         variables = {
-            "first": 1,
+            "first": 25,
             "filters": filters,
         }
         result = self.primitive.session.execute(
@@ -170,6 +208,7 @@ class Files(BaseAction):
         # cleanup memory
         part_data = None
 
+        logger.debug(f"Part {part_number} ETag: {response.headers.get('ETag')}")
         if response.ok:
             # Extract the ETag for the part from the response headers
             etag = response.headers.get("ETag").replace('"', "")
@@ -199,6 +238,8 @@ class Files(BaseAction):
         is_public: bool = False,
         key_prefix: str = "",
         file_id: Optional[str] = None,
+        organization_id: Optional[str] = None,
+        is_global: Optional[bool] = False,
     ):
         if path.exists() is False:
             raise Exception(f"File {path} does not exist.")
@@ -238,13 +279,15 @@ class Files(BaseAction):
                 is_public=is_public,
                 chunk_size=chunk_size,
                 number_of_parts=number_of_parts,
+                organization_id=organization_id,
+                is_global=is_global,
             )
             file_id = pending_file_create.get("id")
             parts_details = pending_file_create.get("partsDetails")
         else:
-            get_file_result = self.get_file(file_id)
+            file_result = self.files(file_id=file_id)
             parts_details = (
-                get_file_result.data.get("files")
+                file_result.data.get("files")
                 .get("edges")[0]
                 .get("node")
                 .get("partsDetails")
@@ -308,6 +351,8 @@ class Files(BaseAction):
         path: Path,
         is_public: bool = False,
         key_prefix: str = "",
+        organization_id: Optional[str] = None,
+        is_global: bool = False,
     ):
         """
         This method uploads a file via the Primitive API.
@@ -319,20 +364,38 @@ class Files(BaseAction):
 
         if is_public:
             operations = (
-                """{ "query": "mutation fileUpload($input: FileUploadInput!) { fileUpload(input: $input) { ... on File { id } } }", "variables": { "input": { "fileObject": null, "isPublic": true, "filePath": \""""
+                """{ "query":\""""
+                + operation_info_fragment.replace("\n", " ")
+                + """mutation fileUpload($input: FileUploadInput!) { fileUpload(input: $input) { ... on File { id }, ...OperationInfoFragment } }", "variables": { "input": { "fileObject": null, "isPublic": true, "filePath": \""""
                 + file_path
                 + """\", "keyPrefix": \""""
                 + key_prefix
-                + """\" } } }"""
+                + """\", "isGlobal": """
+                + ("true" if is_global else "false")
+                + (
+                    f', "organizationId": "{organization_id}"'
+                    if organization_id
+                    else ""
+                )
+                + """ } } }"""
             )  # noqa
 
         else:
             operations = (
-                """{ "query": "mutation fileUpload($input: FileUploadInput!) { fileUpload(input: $input) { ... on File { id } } }", "variables": { "input": { "fileObject": null, "isPublic": false, "filePath": \""""
+                """{ "query":\""""
+                + operation_info_fragment.replace("\n", " ")
+                + """mutation fileUpload($input: FileUploadInput!) { fileUpload(input: $input) { ... on File { id }, ...OperationInfoFragment } }", "variables": { "input": { "fileObject": null, "isPublic": false, "filePath": \""""
                 + file_path
                 + """\", "keyPrefix": \""""
                 + key_prefix
-                + """\" } } }"""
+                + """\", "isGlobal": """
+                + ("true" if is_global else "false")
+                + (
+                    f', "organizationId": "{organization_id}"'
+                    if organization_id
+                    else ""
+                )
+                + """ } } }"""
             )  # noqa
         body = {
             "operations": ("", operations),
@@ -340,14 +403,77 @@ class Files(BaseAction):
             "fileObject": (path.name, open(path, "rb")),
         }
 
-        session = create_requests_session(self.primitive.host_config)
+        session = create_requests_session(host_config=self.primitive.host_config)
         transport = self.primitive.host_config.get("transport")
         url = f"{transport}://{self.primitive.host}/"
         response = session.post(url, files=body)
+
+        if response.ok:
+            messages = (
+                response.json()
+                .get("data", {})
+                .get("fileUpload", {})
+                .get("messages", [])
+            )
+            if messages:
+                raise Exception(
+                    "\n".join([message.get("message") for message in messages])
+                )
+
         return response
 
-    def get_presigned_url(self, file_pk: str):
+    def get_presigned_url(self, file_pk: str) -> str:
         transport = self.primitive.host_config.get("transport")
         host = self.primitive.host_config.get("host")
         file_access_url = f"{transport}://{host}/files/{file_pk}/presigned-url/"
         return file_access_url
+
+    def download_file(
+        self,
+        file_name: str = "",
+        file_id: str = "",
+        organization_id: str = "",
+        organization_slug: str = "",
+        output_path: Path = Path().cwd(),
+        global_file: Optional[bool] = False,
+    ) -> Path:
+        file_pk = None
+        file_size = None
+
+        files_result = self.primitive.files.files(
+            file_id=file_id,
+            file_name=file_name,
+            organization_id=organization_id,
+            organization_slug=organization_slug,
+            global_files=global_file,
+        )
+        if files_data := files_result.data:
+            file = files_data["files"]["edges"][0]["node"]
+            file_pk = file["pk"]
+            file_name = file["fileName"]
+            file_size = int(file["fileSize"])
+
+        if not file_pk:
+            raise Exception(
+                "File not found on remote server. Please check file name or file id"
+            )
+
+        session = create_requests_session(host_config=self.primitive.host_config)
+        transport = self.primitive.host_config.get("transport")
+
+        if file_size and file_size < 5 * 1024 * 1024:
+            url = f"{transport}://{self.primitive.host}/files/{file_pk}/stream/"
+        else:
+            url = f"{transport}://{self.primitive.host}/files/{file_pk}/presigned-url/"
+
+        downloaded_file = output_path / file_name
+
+        with session.get(url, stream=True) as response:
+            response.raise_for_status()
+            with open(downloaded_file, "wb") as partial_downloaded_file:
+                for chunk in response.iter_content(chunk_size=8192):
+                    if chunk:
+                        partial_downloaded_file.write(chunk)
+                        partial_downloaded_file.flush()
+
+        return downloaded_file
