@@ -4,7 +4,11 @@ fragment FileFragment on File {
   pk
   createdAt
   updatedAt
-  createdBy
+  createdBy {
+    id
+    pk
+    username
+  }
   location
   fileName
   fileSize
@@ -13,6 +17,5 @@ fragment FileFragment on File {
   isComplete
   partsDetails
   humanReadableMemorySize
-  contents
 }
 """
