@@ -1191,6 +1191,7 @@ class Hardware(BaseAction):
         is_online: Optional[bool] = None,
         is_rebooting: Optional[bool] = None,
         start_rebooting_at: Optional[str] = None,
+        requires_operating_system_installation: Optional[bool] = None,
     ):
         new_state: dict = {
             "id": hardware_id,
@@ -1201,6 +1202,10 @@ class Hardware(BaseAction):
             new_state["isRebooting"] = is_rebooting
         if start_rebooting_at is not None:
             new_state["startRebootingAt"] = start_rebooting_at
+        if requires_operating_system_installation is not None:
+            new_state["requiresOperatingSystemInstallation"] = (
+                requires_operating_system_installation
+            )
 
         mutation = gql(hardware_update_mutation)
 
