@@ -3,6 +3,7 @@ import shutil
 from enum import Enum
 from pathlib import Path
 from shutil import copy2, copytree, rmtree
+import subprocess
 from typing import Optional
 from urllib.request import urlopen
 
@@ -11,9 +12,11 @@ from gql import gql
 from loguru import logger
 
 from primitive.graphql.relay import from_base64
+from primitive.operating_systems.exceptions import P_CLI_200, P_CLI_201
 from primitive.operating_systems.graphql.mutations import (
     operating_system_create_mutation,
     operating_system_update_mutation,
+    operating_system_delete_mutation,
 )
 from primitive.operating_systems.graphql.queries import operating_system_list_query
 from primitive.utils.actions import BaseAction
@@ -64,6 +67,7 @@ class OperatingSystems(BaseAction):
         is_id = False
         is_slug = False
         id = None
+        slug = None
         # first check if the operating_system_identifier is a slug or ID
         try:
             type_name, id = from_base64(operating_system_identifier)
@@ -78,10 +82,11 @@ class OperatingSystems(BaseAction):
 
         except ValueError:
             is_slug = True
+            slug = operating_system_identifier
 
         return {
             "is_slug": is_slug,
-            "slug": operating_system_identifier,
+            "slug": slug,
             "is_id": is_id,
             "id": id,
         }
@@ -323,13 +328,12 @@ class OperatingSystems(BaseAction):
             self.primitive.operating_systems._is_slug_available(
                 slug=formatted_slug,
                 organization_id=organization_id,
+                is_global=is_global,
             )
         )
 
         if not is_slug_available and not overwrite:
-            raise Exception(
-                f"Operating system with slug {formatted_slug} already exists."
-            )
+            raise P_CLI_200()
 
         is_known_checksum_file_type = (
             checksum_file_type
@@ -482,6 +486,43 @@ class OperatingSystems(BaseAction):
 
         return nodes
 
+    @guard
+    def _delete_query(self, id: str):
+        mutation = gql(operating_system_delete_mutation)
+
+        result = self.primitive.session.execute(
+            mutation, variable_values={"input": {"id": id}}, get_execution_result=True
+        )
+
+        return result.data.get("operatingSystemDelete")
+
+    @guard
+    def delete(
+        self,
+        slug: str = None,
+        id: str = None,
+        organization_id: str = None,
+        is_global: bool = False,
+    ):
+        if not slug and not id:
+            raise Exception("operating_systems.delete - slug or id is required")
+
+        get_args = {}
+
+        if slug:
+            get_args["slug"] = slug
+        else:
+            get_args["id"] = id
+
+        if is_global:
+            get_args["is_global"] = is_global
+        else:
+            get_args["organization_id"] = organization_id
+
+        operating_system = self.primitive.operating_systems.get(**get_args)
+
+        return self._delete_query(id=operating_system["id"])
+
     @guard
     def _get_organization_or_global_operating_system(
         self, organization_id: str, slug: str | None = None, id: str | None = None
@@ -491,7 +532,7 @@ class OperatingSystems(BaseAction):
                 organization_id=organization_id, slug=slug, id=id
             )
             return operating_system, False
-        except self.primitive.operating_systems.OperatingSystemNotFound:
+        except P_CLI_201:
             # If the operating system isn't found in the users org, check the global primitive operating systems
             operating_system = self.primitive.operating_systems.get(
                 slug=slug, id=id, is_global=True
@@ -507,7 +548,7 @@ class OperatingSystems(BaseAction):
                 organization_id=organization_id, slug=slug, id=id
             )
             return operating_system, False
-        except self.primitive.operating_systems.OperatingSystemNotFound:
+        except P_CLI_201:
             # If the operating system isn't found in the users org, check the global primitive operating systems
             operating_system = self.primitive.operating_systems.get(
                 slug=slug, id=id, is_global=True
@@ -613,9 +654,6 @@ class OperatingSystems(BaseAction):
     ):
         shutil.rmtree(f"{get_operating_systems_cache()}/{operating_system_name}")
 
-    class OperatingSystemNotFound(Exception):
-        pass
-
     @guard
     def get(
         self,
@@ -635,27 +673,29 @@ class OperatingSystems(BaseAction):
 
         if len(operating_systems) == 0:
             if slug:
-                raise self.OperatingSystemNotFound(
-                    f"No operating system found for slug {slug}."
-                )
+                raise P_CLI_201(f"No operating system found for slug {slug}.")
             else:
-                raise self.OperatingSystemNotFound(
-                    f"No operating system found for ID {id}."
-                )
+                raise P_CLI_201(f"No operating system found for ID {id}.")
 
         return operating_systems[0]
 
     @guard
-    def _is_slug_available(self, slug: str, organization_id: str):
+    def _is_slug_available(self, slug: str, organization_id: str, is_global: bool):
+        formatted_slug = slugify(slug)
+
         query = gql(operating_system_list_query)
 
-        variables = {
-            "filters": {
-                "slug": {"exact": slug},
-                "organization": {"id": organization_id},
-            }
+        filters = {
+            "slug": {"exact": formatted_slug},
         }
 
+        if is_global:
+            filters["isGlobal"] = {"exact": True}
+        else:
+            filters["organization"] = {"id": organization_id}
+
+        variables = {"filters": filters}
+
         result = self.primitive.session.execute(
             query, variable_values=variables, get_execution_result=True
         )
@@ -803,7 +843,7 @@ class OperatingSystems(BaseAction):
     def _create_boot_script_for_operating_system(
         self,
         target_ip_address: str,
-        controller_hostname: str,
+        file_server_hostname: str,
         operating_system_slug: str,
         iso_filename: str,
     ):
@@ -815,15 +855,15 @@ class OperatingSystems(BaseAction):
 
         if operating_system_slug.startswith("ubuntu"):
             boot_script_content = f"""#!ipxe
-kernel http://{controller_hostname}/pxe/operating-systems/{operating_system_slug}/vmlinuz
-initrd http://{controller_hostname}/pxe/operating-systems/{operating_system_slug}/initrd
-imgargs vmlinuz initrd=initrd ip=dhcp BOOTIF=${{netX/mac}} cloud-config-url=/dev/null priority=critical DEBCONF_DEBUG=5 url=http://{controller_hostname}/pxe/operating-systems/{operating_system_slug}/{iso_filename} autoinstall ds=nocloud-net;s=http://{controller_hostname}/pxe/autoinstall/
+kernel http://{file_server_hostname}/pxe/operating-systems/{operating_system_slug}/vmlinuz
+initrd http://{file_server_hostname}/pxe/operating-systems/{operating_system_slug}/initrd
+imgargs vmlinuz initrd=initrd ip=dhcp BOOTIF=${{netX/mac}} cloud-config-url=/dev/null priority=critical DEBCONF_DEBUG=5 url=http://{file_server_hostname}/pxe/operating-systems/{operating_system_slug}/{iso_filename} autoinstall ds=nocloud-net;s=http://{file_server_hostname}/pxe/hardware/{target_ip_address}/autoinstall/
 boot
 """
         elif operating_system_slug.startswith("rocky"):
             boot_script_content = f"""#!ipxe
-kernel http://{controller_hostname}/pxe/operating-systems/{operating_system_slug}/vmlinuz ip=dhcp BOOTIF=${{netX/mac}} inst.stage2=http://{controller_hostname}/pxe/operating-systems/{operating_system_slug}/os/ inst.ks=http://{controller_hostname}/pxe/kickstart/rocky10.ks
-initrd http://{controller_hostname}/pxe/operating-systems/{operating_system_slug}/initrd.img
+kernel http://{file_server_hostname}/pxe/operating-systems/{operating_system_slug}/vmlinuz ip=dhcp BOOTIF=${{netX/mac}} inst.stage2=http://{file_server_hostname}/pxe/operating-systems/{operating_system_slug}/os/ inst.ks=http://{file_server_hostname}/pxe/kickstart/rocky10.ks
+initrd http://{file_server_hostname}/pxe/operating-systems/{operating_system_slug}/initrd.img
 boot
 """
 
@@ -837,20 +877,131 @@ boot
 
         logger.info(f"Created PXE boot script at {boot_script_path}")
 
+    def _create_autoinstall_config(
+        self,
+        file_server_hostname: str,
+        target_ip_address: str,
+        target_username: str,
+        target_password: str,
+        target_machine_name: str,
+        target_auth_token: str,
+    ):
+        pxe_os_path = Path(f"/var/www/html/pxe/hardware/{target_ip_address}")
+        autoinstall_path = Path(pxe_os_path / "autoinstall")
+        autoinstall_path.mkdir(parents=True, exist_ok=True)
+
+        meta_data_file = Path(autoinstall_path / "meta-data")
+
+        if meta_data_file.exists():
+            meta_data_file.unlink()
+        with open(meta_data_file, "w") as meta_data:
+            meta_data.write(
+                f"instance-id: iid-local01\nlocal-hostname: {target_machine_name}\n"
+            )
+        logger.info(f"Created autoinstall meta-data at {meta_data_file}")
+
+        user_data_file = Path(autoinstall_path / "user-data")
+        if user_data_file.exists():
+            user_data_file.unlink()
+
+        hashed_password = (
+            subprocess.check_output(["mkpasswd", "--method=SHA-512", target_password])
+            .decode()
+            .strip()
+        )
+
+        with open(user_data_file, "w") as user_data:
+            user_data.write(f"""#cloud-config
+autoinstall:
+  version: 1
+  locale: en_US.UTF-8
+  keyboard:
+    layout: us
+  timezone: America/New_York
+  user-data:
+    locale: en_US.UTF-8
+    disable_root: false
+    users:
+      - name: {target_username}
+        shell: /bin/bash
+        groups: [adm, sudo]
+        sudo: ALL=(ALL) NOPASSWD:ALL
+    runcmd:
+      - curl -LsSf http://{file_server_hostname}/pxe/hardware/{target_ip_address}/autoinstall/user-data-script.sh | sh
+
+  identity:
+    hostname: {target_machine_name}
+    username: {target_username}
+    password: {hashed_password}
+
+  apt:
+    disable_suites: [updates, security]
+
+  storage:
+    swap:
+      size: 0
+    layout:
+      name: direct
+      match:
+        size: smallest
+
+  packages:
+    - curl
+
+  ssh:
+    install-server: true
+    allow-pw: true
+""")
+
+        user_data_script_file = Path(autoinstall_path / "user-data-script.sh")
+        if user_data_script_file.exists():
+            user_data_script_file.unlink()
+        with open(user_data_script_file, "w") as user_data_script:
+            user_data_script.write(f"""#! /bin/bash
+# LOGS ARE LOCATED AT var/log/cloud-init-output.log
+echo $USER
+echo $ENV
+
+curl -LsSf https://astral.sh/uv/install.sh | sh
+source $HOME/.local/bin/env
+source $HOME/.bashrc
+uv python install --default 3.14.0
+uv venv
+uv init
+source $HOME/.venv/bin/activate
+uv add --no-cache primitive
+primitive --host api.dev.primitive.tech config --auth-token {target_auth_token}
+primitive --host api.dev.primitive.tech hardware register --issue-certificate
+primitive --debug --host api.dev.primitive.tech daemons install
+""")
+
     def _prepare_operating_system_for_pxe(
         self,
         organization_id: str,
         operating_system_slug: str,
         iso_filename: str,
-        controller_hostname: str,
+        file_server_hostname: str,
         target_ip_address: str,
+        target_username: str,
+        target_password: str,
+        target_machine_name: str,
+        target_auth_token: str,
     ):
         if not target_ip_address:
             raise Exception("Target IP address is required for PXE setup.")
 
+        self._create_autoinstall_config(
+            file_server_hostname=file_server_hostname,
+            target_ip_address=target_ip_address,
+            target_username=target_username,
+            target_password=target_password,
+            target_machine_name=target_machine_name,
+            target_auth_token=target_auth_token,
+        )
+
         self._create_boot_script_for_operating_system(
             target_ip_address=target_ip_address,
-            controller_hostname=controller_hostname,
+            file_server_hostname=file_server_hostname,
             operating_system_slug=operating_system_slug,
             iso_filename=iso_filename,
         )
