@@ -46,3 +46,19 @@ mutation operatingSystemUpdate($input: OperatingSystemUpdateInput!) {
 }
 """
 )
+
+operating_system_delete_mutation = (
+    operation_info_fragment
+    + """
+mutation operatingSystemDelete($input: NodeInput!) {
+    operatingSystemDelete(input: $input) {
+        ... on OperatingSystem {
+            id
+            pk
+            slug
+        }
+        ...OperationInfoFragment
+    }
+}
+"""
+)
