@@ -67,3 +67,9 @@ class Organizations(BaseAction):
         )
         organizations = [edge["node"] for edge in result.data["organizations"]["edges"]]
         return organizations[0]
+
+    @guard
+    def get_default_organization(self):
+        whoami_result = self.primitive.auth.whoami()
+        default_organization = whoami_result.data["whoami"]["defaultOrganization"]
+        return default_organization
