@@ -1,5 +1,5 @@
 authorized_keys_query = """
-query authorizedKeys($reservationId: GlobalID!) {
+query authorizedKeys($reservationId: ID!) {
     authorizedKeys(reservationId: $reservationId)
 }
 """
